//+------------------------------------------------------------------+
//|                                               Kalman+Filters.mq5 |
//|                                              Copyright 2017, DNG |
//|                                 http://www.mql5.com/en/users/dng |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, DNG"
#property link      "http://www.mql5.com/en/users/dng"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
//--- available signals
#include <Expert\Signal\SignalKalman+Filters.mqh>
//--- available trailing
#include <Expert\Trailing\TrailingNone.mqh>
//--- available money management
#include <Expert\Money\MoneyFixedLot.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string             Expert_Title                         ="Kalman+Filters"; // Document name
ulong                    Expert_MagicNumber                   =7150;             // 
bool                     Expert_EveryTick                     =false;            // 
//--- inputs for main signal
input int                Signal_ThresholdOpen                 =10;               // Signal threshold value to open [0...100]
input int                Signal_ThresholdClose                =10;               // Signal threshold value to close [0...100]
input double             Signal_PriceLevel                    =0.0;              // Price level to execute a deal
input double             Signal_StopLevel                     =50.0;             // Stop Loss level (in points)
input double             Signal_TakeLevel                     =50.0;             // Take Profit level (in points)
input int                Signal_Expiration                    =4;                // Expiration of pending orders (in bars)
input ENUM_TIMEFRAMES    Signal_Kalman_Filter_TimeFrame       =PERIOD_H1;        // Signals of Kalman's filter degign by DNG Timeframe
input uint               Signal_Kalman_Filter_HistoryBars     =3000;             // Signals of Kalman's filter degign by DNG Bars in history to analysis
input uint               Signal_Kalman_Filter_ShiftPeriod     =0;                // Signals of Kalman's filter degign by DNG Period for shift
input uint               Signal_Kalman_Filter_JAW_Period      =13;               // Signals of Kalman's filter degign by DNG JAW Period
input uint               Signal_Kalman_Filter_JAW_Shift       =8;                // Signals of Kalman's filter degign by DNG JAW Shift
input uint               Signal_Kalman_Filter_TEETH_Period    =8;                // Signals of Kalman's filter degign by DNG TEETH Period
input uint               Signal_Kalman_Filter_TEETH_Shift     =5;                // Signals of Kalman's filter degign by DNG TEETH Shift
input uint               Signal_Kalman_Filter_LIPS_Period     =5;                // Signals of Kalman's filter degign by DNG LIPS Period
input uint               Signal_Kalman_Filter_LIPS_Shift      =3;                // Signals of Kalman's filter degign by DNG LIPS_Shift
input ENUM_MA_METHOD     Signal_Kalman_Filter_Alligator_Method=MODE_SMMA;        // Signals of Kalman's filter degign by DNG Method
input ENUM_APPLIED_PRICE Signal_Kalman_Filter_Alligator_Price =PRICE_MEDIAN;     // Signals of Kalman's filter degign by DNG Alligator Price
input uint               Signal_Kalman_Filter_MACD_Fast       =12;               // Signals of Kalman's filter degign by DNG MACD Fast
input uint               Signal_Kalman_Filter_MACD_Slow       =26;               // Signals of Kalman's filter degign by DNG MACD Slow
input uint               Signal_Kalman_Filter_MACD_Signal     =9;                // Signals of Kalman's filter degign by DNG MACD Signal
input ENUM_APPLIED_PRICE Signal_Kalman_Filter_MACD_Price      =PRICE_CLOSE;      // Signals of Kalman's filter degign by DNG MACD Price-----------------------------------------------------
input double             Signal_Kalman_Filter_Weight          =1.0;              // Signals of Kalman's filter degign by DNG Weight [0...1.0]
//--- inputs for money
input double             Money_FixLot_Percent                 =10.0;             // Percent
input double             Money_FixLot_Lots                    =0.1;              // Fixed volume
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CExpert ExtExpert;
//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- Initializing expert
   if(!ExtExpert.Init(Symbol(),Period(),Expert_EveryTick,Expert_MagicNumber))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing expert");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Creating signal
   CExpertSignal *signal=new CExpertSignal;
   if(signal==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating signal");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//---
   ExtExpert.InitSignal(signal);
   signal.ThresholdOpen(Signal_ThresholdOpen);
   signal.ThresholdClose(Signal_ThresholdClose);
   signal.PriceLevel(Signal_PriceLevel);
   signal.StopLevel(Signal_StopLevel);
   signal.TakeLevel(Signal_TakeLevel);
   signal.Expiration(Signal_Expiration);
//--- Creating filter CSignalKalman
   CSignalKalman *filter0=new CSignalKalman;
   if(filter0==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter0");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter0);
//--- Set filter parameters
   filter0.TimeFrame(Signal_Kalman_Filter_TimeFrame);
   filter0.HistoryBars(Signal_Kalman_Filter_HistoryBars);
   filter0.ShiftPeriod(Signal_Kalman_Filter_ShiftPeriod);
   filter0.JAW_Period(Signal_Kalman_Filter_JAW_Period);
   filter0.JAW_Shift(Signal_Kalman_Filter_JAW_Shift);
   filter0.TEETH_Period(Signal_Kalman_Filter_TEETH_Period);
   filter0.TEETH_Shift(Signal_Kalman_Filter_TEETH_Shift);
   filter0.LIPS_Period(Signal_Kalman_Filter_LIPS_Period);
   filter0.LIPS_Shift(Signal_Kalman_Filter_LIPS_Shift);
   filter0.Alligator_Method(Signal_Kalman_Filter_Alligator_Method);
   filter0.Alligator_Price(Signal_Kalman_Filter_Alligator_Price);
   filter0.MACD_Fast(Signal_Kalman_Filter_MACD_Fast);
   filter0.MACD_Slow(Signal_Kalman_Filter_MACD_Slow);
   filter0.MACD_Signal(Signal_Kalman_Filter_MACD_Signal);
   filter0.MACD_Price(Signal_Kalman_Filter_MACD_Price);
   filter0.Weight(Signal_Kalman_Filter_Weight);
//--- Creation of trailing object
   CTrailingNone *trailing=new CTrailingNone;
   if(trailing==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert.InitTrailing(trailing))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Set trailing parameters
//--- Creation of money object
   CMoneyFixedLot *money=new CMoneyFixedLot;
   if(money==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert.InitMoney(money))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Set money parameters
   money.Percent(Money_FixLot_Percent);
   money.Lots(Money_FixLot_Lots);
//--- Check all trading objects parameters
   if(!ExtExpert.ValidationSettings())
     {
      //--- failed
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Tuning of all necessary indicators
   if(!ExtExpert.InitIndicators())
     {
      //--- failed
      printf(__FUNCTION__+": error initializing indicators");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- ok
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   ExtExpert.Deinit();
  }
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
  {
   ExtExpert.OnTick();
  }
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
   ExtExpert.OnTrade();
  }
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
   ExtExpert.OnTimer();
  }
//+------------------------------------------------------------------+
